package pkg

import (
	"context"
	"errors"
	"net/http"
	"net/url"
	"strings"
)

const (
	ContextCompanyKey = "numerus-company"
)

type Company struct {
	Id   int
	Slug string
}

func CompanyHandler(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		slug := r.URL.Path
		if idx := strings.IndexByte(slug, '/'); idx >= 0 {
			slug = slug[:idx]
		}

		conn := getConn(r)
		company := &Company{
			Slug: slug,
		}
		err := conn.QueryRow(r.Context(), "select company_id from company where slug = $1", slug).Scan(&company.Id)
		if err != nil {
			http.NotFound(w, r)
			return
		}
		ctx := context.WithValue(r.Context(), ContextCompanyKey, company)
		r = r.WithContext(ctx)

		// Same as StripPrefix
		p := strings.TrimPrefix(r.URL.Path, slug)
		rp := strings.TrimPrefix(r.URL.RawPath, slug)
		if len(p) < len(r.URL.Path) && (r.URL.RawPath == "" || len(rp) < len(r.URL.RawPath)) {
			r2 := new(http.Request)
			*r2 = *r
			r2.URL = new(url.URL)
			*r2.URL = *r.URL
			if p == "" {
				r2.URL.Path = "/"
			} else {
				r2.URL.Path = p
			}
			r2.URL.RawPath = rp
			next.ServeHTTP(w, r2)
		} else {
			http.NotFound(w, r)
		}
	})
}

func getCompany(r *http.Request) *Company {
	company := r.Context().Value(ContextCompanyKey)
	if company == nil {
		return nil
	}
	return company.(*Company)
}

type TaxDetailsPage struct {
	Title        string
	BusinessName string
	VATIN        string
	TradeName    string
	Phone        string
	Email        string
	Web          string
	Address      string
	City         string
	Province     string
	PostalCode   string
	Country      string
}

func CompanyTaxDetailsHandler() http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		locale := getLocale(r)
		page := &TaxDetailsPage{
			Title: pgettext("title", "Tax Details", locale),
		}
		company := mustGetCompany(r)
		conn := getConn(r)
		if r.Method == "POST" {
			r.ParseForm()
			page.BusinessName = r.FormValue("business_name")
			page.VATIN = r.FormValue("vatin")
			page.TradeName = r.FormValue("trade_name")
			page.Phone = r.FormValue("phone")
			page.Email = r.FormValue("email")
			page.Web = r.FormValue("web")
			page.Address = r.FormValue("address")
			page.City = r.FormValue("city")
			page.Province = r.FormValue("province")
			page.PostalCode = r.FormValue("postal_code")
			page.Country = r.FormValue("country")
			conn.MustExec(r.Context(), "update company set business_name = $1, vatin = $2, trade_name = $3, phone = $4, email = $5, web = $6, address = $7, city = $8, province = $9, postal_code = $10, country = $11 where company_id = $12", page.BusinessName, page.VATIN, page.TradeName, page.Phone, page.Email, page.Web, page.Address, page.City, page.Province, page.PostalCode, page.Country, company.Id)
			http.Redirect(w, r, "/company/"+company.Slug+"/tax-details", http.StatusSeeOther)
		} else {
			err := conn.QueryRow(r.Context(), "select business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country from company where company_id = $1", company.Id).Scan(&page.BusinessName, &page.VATIN, &page.TradeName, &page.Phone, &page.Email, &page.Web, &page.Address, &page.City, &page.Province, &page.PostalCode, &page.Country)
			if err != nil {
				panic(err)
			}
		}
		mustRenderAppTemplate(w, r, "tax-details.html", page)
	})
}

func mustGetCompany(r *http.Request) *Company {
	company := getCompany(r)
	if company == nil {
		panic(errors.New("company: required but not found"))
	}
	return company
}
